"""
# Copyright (C) 2013-2016 Stray <stray411@hotmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For questions regarding this module contact
# Stray <stray411@hotmail.com>
"""

# emacs-mode: -*- python-*-
# -*- coding: utf-8 -*-

from __future__ import with_statement
import Live 
import sys
from _Framework.ControlSurface import ControlSurface  
from _Framework.InputControlElement import * 
from _Framework.ButtonElement import ButtonElement  
from _Framework.ButtonMatrixElement import ButtonMatrixElement  
from _Framework.EncoderElement import EncoderElement
from _Framework.TransportComponent import TransportComponent  
from _Framework.SessionZoomingComponent import DeprecatedSessionZoomingComponent as SessionZoomingComponent
from SMSession import SMSession
from SMMixer import SMMixer  
from SMMisc import SMMisc
from SMDevice import SMDevice

IS_MTY = True
SEP_COUNT = 8
SCRIPT_NAME = 'Chameleon Script v1.8'
FOLDER_NAME = '/Chameleon/'

class SMMain(ControlSurface):
    __module__ = __name__
    __doc__ = " Control Surface script for Live 10 "
    
    def __init__(self, c_instance):
        ControlSurface.__init__(self, c_instance)
	
	self._globals = {'TRACK_COUNT': 8, 
	                 'SCENE_COUNT': 8,
	                 
	                 'REINIT_NOTE_NUMBER': 0,
	                 
	                 'CL_LAUNCH_LED_VALUE': 127,
	                 'CL_PLAY_LED_VALUE': 126,
	                 'CL_TRIG_RECORD_VALUE': 125,
	                 'CL_RECORD_VALUE': 124,
	                 'CL_LOOP_VALUE': 123, 
	                 'CL_SELECTABLE_VALUE': 1,
	                 
	                 'BPM_ENC_RES': 1.0,
	                 'BPM_ENC_SHIFT_RES': 0.1,
	                 'BPM_MAX': 999.0,
	                 'CLIP_LOOP_RES': 1.0,
	                 'CLIP_LOOP_SHIFT_RES': 0.625}
	self.get_user_settings()
	with self.component_guard():
	    self._shift_button = ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 102)
	    self._mixer = SMMixer(self, self._globals['TRACK_COUNT'])
	    self._setup_mixer_control()
	    self._session = SMSession(self._globals['TRACK_COUNT'], self._globals['SCENE_COUNT']) 
	    self._setup_session_control()
	    self._transport = TransportComponent()
	    self._setup_transport_control()
	    self._device = SMDevice()
	    self._setup_device_control()
	    self._misc = SMMisc(self)
	    self._setup_misc_control()
	    self.set_highlighting_session_component(self._session)

	live = Live.Application.get_application()
        self.log_message('SM LOG ------- Script: ' + str(SCRIPT_NAME) + ' ------- Live Version: ' + str(live.get_major_version()) + '.' + str(live.get_minor_version()) + '.' + str(live.get_bugfix_version()) + ' -------')
        

    def disconnect(self):
	self._shift_button = None
        self._session = None
	self._mixer = None
	self._transport = None
	self._device = None
	self._misc = None
        ControlSurface.disconnect(self)
      
	
    def _setup_mixer_control(self):
	""" This sets up volume, pan, sends A-D, EQ Low/Mid/Hi, select, mute, solo, arm, master select, master volume, master pan and cue volume.  It also assigns shift button for use with overriding prefs options for solo/arm exclusive. 
	Volume - Tr1-8   = Ch15, CC0-7
	Pan -    Tr1-8   = Ch15, CC8-15
	Send A - Tr1-8   = Ch15, CC16-23
	Send B - Tr1-8   = Ch15, CC24-31
	Send C - Tr1-8   = Ch15, CC32-39
	Send D - Tr1-8   = Ch15, CC40-47
	EQ Low - Tr1-8   = Ch15, CC48-55
	EQ Mid - Tr1-8   = Ch15, CC56-63
	EQ Hi  - Tr1-8   = Ch15, CC64-71
	Select - Tr1-8   = Ch15, CC80-87
	Mute   - Tr1-8   = Ch15, CC88-95
	Solo   - Tr1-8   = Ch15, CC96-103
	Arm    - Tr1-8   = Ch15, CC104-111
	
	Master Select    = Ch16, CC103 
	Master Volume    = Ch16, CC104 
	Master Pan       = Ch16, CC105 
	Cue Volume       = Ch16, CC106 
	"""
	self._mixer.set_track_offset(0)  
        self.song().view.selected_track = self._mixer.channel_strip(0)._track 
	self._mixer.selected_strip().set_track(self.song().view.selected_track)
	
        for track in range(self._globals['TRACK_COUNT']):
            strip = self._mixer.channel_strip(track)
	    strip.set_volume_control(EncoderElement(MIDI_CC_TYPE, 14, track, Live.MidiMap.MapMode.absolute))
	    strip.set_pan_control(EncoderElement(MIDI_CC_TYPE, 14, track + 8, Live.MidiMap.MapMode.absolute))
	    strip.set_send_controls(tuple(EncoderElement(MIDI_CC_TYPE, 14, ((index * SEP_COUNT) + track + 16), Live.MidiMap.MapMode.absolute) for index in range(4)))
	    eq = self._mixer.track_eq(track)
            eq.set_gain_controls(tuple(EncoderElement(MIDI_CC_TYPE, 14, ((index * SEP_COUNT) + track + 48), Live.MidiMap.MapMode.absolute) for index in range(3)))
            strip.set_select_button(ButtonElement(IS_MTY, MIDI_CC_TYPE, 14, (80 + track)))
            strip.set_mute_button(ButtonElement(IS_MTY, MIDI_CC_TYPE, 14, (88 + track)))
            strip.set_solo_button(ButtonElement(IS_MTY, MIDI_CC_TYPE, 14, (96 + track)))
            strip.set_arm_button(ButtonElement(IS_MTY, MIDI_CC_TYPE, 14, (104 + track)))
            strip.set_invert_mute_feedback(True)
	    strip.set_shift_button(self._shift_button)
	    
	self._mixer.master_strip().set_select_button(ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 103))
	self._mixer.master_strip().set_volume_control(EncoderElement(MIDI_CC_TYPE, 15, 104, Live.MidiMap.MapMode.absolute))
	self._mixer.master_strip().set_pan_control(EncoderElement(MIDI_CC_TYPE, 15, 105, Live.MidiMap.MapMode.absolute))
	self._mixer.set_prehear_volume_control(EncoderElement(MIDI_CC_TYPE, 15, 106, Live.MidiMap.MapMode.absolute))
     

    def _setup_session_control(self):
	""" This sets up clip launch matrix, bank buttons, track stop buttons, scene launch buttons and stop all button. It also assigns shift button for use with session overview. 
	Clip Slot 1 - Tr1-8 = Ch16, CC0-7 
	Clip Slot 2 - Tr1-8 = Ch16, CC8-15
	Clip Slot 3 - Tr1-8 = Ch16, CC16-23
	Clip Slot 4 - Tr1-8 = Ch16, CC24-31
	Clip Slot 5 - Tr1-8 = Ch16, CC32-39
	Clip Slot 6 - Tr1-8 = Ch16, CC40-47
	Clip Slot 7 - Tr1-8 = Ch16, CC48-55
	Clip Slot 8 - Tr1-8 = Ch16, CC56-63
	
	Scn Launch 1-8      = Ch16, CC64-71
	Left/Right/Down/Up  = Ch16, CC85-88
	Stop All Clips      = Ch16, CC97
	
	Track Stop - Tr1-8  = Ch15, CC72-79 
	"""
        self._session.set_offsets(0, 0)  
	self._session.set_mixer(self._mixer)
	
	left_button = ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 85)
        right_button = ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 86)
        down_button = ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 87)
	up_button = ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 88)        
        self._session.set_track_bank_buttons(right_button, left_button)
        self._session.set_scene_bank_buttons(down_button, up_button)  
	
	track_stop_buttons = [ ButtonElement(IS_MTY, MIDI_CC_TYPE,  14, (index + 72)) for index in range(self._globals['TRACK_COUNT']) ]
	self._session.set_stop_track_clip_buttons(tuple(track_stop_buttons))
        self._session.set_stop_all_clips_button(ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 97))
	
	matrix = ButtonMatrixElement()   
        scene_launch_buttons = [ ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, (index + 64)) for index in range(self._globals['SCENE_COUNT']) ]
	
        for scene_index in range(self._globals['SCENE_COUNT']):
            scene = self._session.scene(scene_index)
            scene.set_launch_button(scene_launch_buttons[scene_index])
	    button_row = []
            for track_index in range(self._globals['TRACK_COUNT']):
                button = ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, ((scene_index * SEP_COUNT) + track_index))
                button_row.append(button)
                clip_slot = scene.clip_slot(track_index)
                clip_slot.set_triggered_to_play_value(self._globals['CL_LAUNCH_LED_VALUE'])
                clip_slot.set_triggered_to_record_value(self._globals['CL_TRIG_RECORD_VALUE'])
                clip_slot.set_stopped_value(self._globals['CL_SELECTABLE_VALUE'])
                clip_slot.set_started_value(self._globals['CL_PLAY_LED_VALUE'])
		clip_slot.set_looping_value(self._globals['CL_LOOP_VALUE'])
                clip_slot.set_recording_value(self._globals['CL_RECORD_VALUE'])
                clip_slot.set_launch_button(button)
            matrix.add_row(tuple(button_row))

        session_zoom = SessionZoomingComponent(self._session)
        session_zoom.set_button_matrix(matrix)
        session_zoom.set_zoom_button(self._shift_button) 
        session_zoom.set_nav_buttons(up_button, down_button, left_button, right_button)
        session_zoom.set_scene_bank_buttons(tuple(scene_launch_buttons))
        session_zoom.set_stopped_value(self._globals['CL_SELECTABLE_VALUE'])
	session_zoom.set_playing_value(self._globals['CL_PLAY_LED_VALUE'])
        session_zoom.set_selected_value(self._globals['CL_RECORD_VALUE'])	
        self._set_session_highlight(0, 0, self._globals['TRACK_COUNT'], self._globals['SCENE_COUNT'], True)# This ensures that the highlight can include return tracks on init.
	
    
    def _setup_transport_control(self):
	""" This sets up play, stop, rec, tap tempo and nudge -/+.
	Play      = Ch16, CC94
	Stop      = Ch16, CC95
	Rec       = Ch16, CC96
	Tap       = Ch16, CC98
	Nudge -/+ = Ch16, CC99-100
	"""
	self._transport.set_play_button(ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 94))
	self._transport.set_stop_button(ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 95))
        self._transport.set_record_button(ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 96))
	self._transport.set_tap_tempo_button(ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 98))
	self._transport.set_nudge_buttons(ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 100), ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 99))
	
     
    def _setup_device_control(self):
	""" This sets up device encoders, device bank -/+ and device on/off. 
	Device Encoders  = Ch16, CC72-79
	Device Bank -/+  = Ch16, CC80-81
	Device I/O       = Ch16, CC84
	"""
        self.set_device_component(self._device)
	self._device.set_parameter_controls(tuple(EncoderElement(MIDI_CC_TYPE, 15, 72 + index, Live.MidiMap.MapMode.absolute) for index in range(8)))
        self._device.set_bank_nav_buttons(ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 80), ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 81)) 
	self._device.set_on_off_button(ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 84))
    
	
    def _setup_misc_control(self):
	""" This sets up device left/right, clip loop start -/+, clip loop end -/+, clip loop on/off, tempo encoder and reinit function.  It also assigns shift button for use with modifying resolution.
	Device Left/Right   = Ch16, CC82-83
	Clip Loop Start -/+ = Ch16, CC89-90
	Clip Loop End -/+   = Ch16, CC91-92
	Clip Loop I/O       = Ch16, CC93
	Tempo Encoder       = Ch16, CC101
	
	Reinit              = Ch16, Note0 (note number can be set via variables).
	"""
	self._misc.set_shift_button(self._shift_button)
	self._misc.set_device_left_button(ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 82))
	self._misc.set_device_right_button(ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 83))
	self._misc.set_clip_loop_start_dec_button(ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 89))
	self._misc.set_clip_loop_start_inc_button(ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 90))
	self._misc.set_clip_loop_end_dec_button(ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 91))
	self._misc.set_clip_loop_end_inc_button(ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 92))
	self._misc.set_clip_loop_on_off_button(ButtonElement(IS_MTY, MIDI_CC_TYPE, 15, 93))
	self._misc.set_reinit_button(ButtonElement(IS_MTY, MIDI_NOTE_TYPE, 15, self._globals['REINIT_NOTE_NUMBER']))
	self._misc.set_tempo_encoder(EncoderElement(MIDI_CC_TYPE, 15, 101, Live.MidiMap.MapMode.relative_smooth_two_compliment))
    
            
    def _on_selected_track_changed(self):
	""" On selected track changes, select the last selected device on the track or, if no last selected, select the first device on the track. """
        ControlSurface._on_selected_track_changed(self)        
        track = self.song().view.selected_track
        device_to_select = track.view.selected_device
        if device_to_select == None and len(track.devices) > 0:
            device_to_select = track.devices[0]
        if device_to_select: 
            self.song().view.select_device(device_to_select)
	if self._device_component:
	    self._device_component.set_device(device_to_select)
	   
	
    def get_user_settings(self):
	""" Read prefs file, handle input errors and store results in global dict. """
	try:
	    mrs_path = ''
	    for path in sys.path:
		if 'MIDI Remote Scripts' in path:
		    mrs_path = path
		    break
	    prefs_file = mrs_path + FOLDER_NAME + 'UserPreferences.txt'	    
	    for line in open(prefs_file):
		if not line.startswith(('#', '"', '*')) and '=' in line:
		    formatted_line = str(line).upper()	
		    if '#' in formatted_line:
			formatted_line = formatted_line[0:formatted_line.index('#')].strip()
		    if formatted_line:
			data = formatted_line.split('=')
			if len(data) == 2:
			    key = data[0].strip()
			    value = data[1].strip()
			    self.log_message('SM LOG ------- Current Key: ' + str(key) + '=' + str(value) + ' ------- ')
			    if key in ('TRACK_COUNT', 'SCENE_COUNT'):
				try: int_value = int(value)
				except: int_value = 8
				if int_value in range(1, 9): 
				    self._globals[key] = int_value
			    elif key == 'REINIT_NOTE_NUMBER':
				try: int_value = int(value)
				except: int_value = 0
				if int_value in range(0, 128): 
				    self._globals[key] = int_value
			    elif key.startswith('CL_') and key.endswith('VALUE'):
				try: int_value = int(value)
				except: int_value = 127
				if int_value in range(1, 128): 
				    self._globals[key] = int_value
			    elif '_RES' in key:
				try: float_value = float(value)
				except: float_value = 1.0
				if float_value >= 0.0:
				    self._globals[key] = float_value
			    elif key == 'BPM_MAX':
				try: float_value = float(value)
				except: float_value = 999.0
				if float_value > 20.0 and float_value <= 999.0:
				    self._globals[key] = float_value
	except: pass
	
	
    # >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> General-purpose functions for setting/disconnecting controls <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<	    
    def set_button(self, button, button_var, value_listener, identify_sender=False, function_to_call=None, args=None):
	""" General-purpose set button function that can optionally call a function on successful set. """
	assert button == None or isinstance(button, ButtonElement)
	assert (dir(value_listener).count('im_func') is 1)
	assert isinstance(identify_sender, (int, type(False)))
	assert function_to_call == None or (dir(function_to_call).count('im_func') is 1)
	if button != button_var:
	    if button_var and button_var.value_has_listener(value_listener):
		button_var.remove_value_listener(value_listener)
		button_var.reset()
	    if button:
		if not button.value_has_listener(value_listener):
		    if identify_sender:
			button.add_value_listener(value_listener, True)
		    else:
			button.add_value_listener(value_listener)
		if function_to_call:
		    if args != None:
			function_to_call(args)
		    else:
			function_to_call()
	    button_var = button
	return button_var
    
    
    def set_encoder(self, control, control_var, value_listener=None, identify_sender=False, function_to_call=None, args=None):
	""" General-purpose set encoder function that can optionally call a function on successful set. """
        assert control == None or isinstance(control, EncoderElement)
	assert value_listener == None or (dir(value_listener).count('im_func') is 1)
	assert isinstance(identify_sender, (int, type(False)))
	assert function_to_call == None or (dir(function_to_call).count('im_func') is 1)
	if control != control_var:
	    if control_var and value_listener and control_var.value_has_listener(value_listener):
		control_var.remove_value_listener(value_listener)
		control_var.release_parameter()
	    if control and value_listener and not control.value_has_listener(value_listener):
		if identify_sender:
		    control.add_value_listener(value_listener, True)
		else:
		    control.add_value_listener(value_listener)
	    control_var = control
	    if function_to_call:
		if args != None:
		    function_to_call(args)
		else:
		    function_to_call()
	return control_var
   
    
    def disconnect_button(self, button, value_listener, reset=False):
	""" General-purpose button disconnect that removes listener and returns None. This is also used to disconnect parameter/parameter listeners. """
	assert button == None or isinstance(button, (Live.DeviceParameter.DeviceParameter, ButtonElement))
	assert value_listener == None or (dir(value_listener).count('im_func') is 1) or (dir(value_listener).count('func_name') is 1)
	assert isinstance(reset, (int, type(False)))
	if button:
	    if button.value_has_listener(value_listener):
		button.remove_value_listener(value_listener)
	    if isinstance(button, ButtonElement) and reset:
		button.reset()		
	return None
    
    
    def disconnect_encoder(self, encoder, value_listener=None):
	""" General-purpose encoder disconnect that can just release parameter or can optionally remove value listener too. """
	assert encoder == None or isinstance(encoder, EncoderElement)
	assert value_listener == None or (dir(value_listener).count('im_func') is 1) or (dir(value_listener).count('func_name') is 1)
	if encoder:
	    if value_listener and encoder.value_has_listener(value_listener):
		encoder.remove_value_listener(value_listener)
	    encoder.release_parameter()
	return None   
    
# local variables:
# tab-width: 4
                