"""
# Copyright (C) 2013-2016 Stray <stray411@hotmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For questions regarding this module contact
# Stray <stray411@hotmail.com>
"""

# emacs-mode: -*- python-*-
# -*- coding: utf-8 -*-

import Live 
from _Framework.ControlSurfaceComponent import ControlSurfaceComponent
from _Framework.ButtonElement import ButtonElement
from _Framework.EncoderElement import EncoderElement

class SMMisc(ControlSurfaceComponent):
    __module__ = __name__
    __doc__ = " Miscellaneous mappings for SM script. Includes Device Left/Right, Clip Start Dec/Inc, Clip End Dec/Inc, Clip Loop On/Off, Tempo Dec/Inc and an initialization function. "
    
    def __init__(self, parent):
        ControlSurfaceComponent.__init__(self)
	self._parent = parent
	self._shift_button = None
	self._shift_pressed = False
	self._device_left_button = None
	self._device_right_button = None
	self._clip_loop_start_dec_button = None
	self._clip_loop_start_inc_button = None
	self._clip_loop_end_dec_button = None
	self._clip_loop_end_inc_button = None
	self._clip_loop_on_off_button = None
	self._reinit_button = None
	self._tempo_encoder = None
	self._clip = None
	self.song().view.add_detail_clip_listener(self.on_clip_changed)
	self.on_clip_changed()	
	
	
    def disconnect(self):
	self._shift_button = self._parent.disconnect_button(self._shift_button, self._shift_value)
	self._device_left_button = self._parent.disconnect_button(self._device_left_button, self._device_nav_value)
	self._device_right_button = self._parent.disconnect_button(self._device_right_button, self._device_nav_value)
	self._clip_loop_start_dec_button = self._parent.disconnect_button(self._clip_loop_start_dec_button, self._clip_loop_start_value)
	self._clip_loop_start_inc_button = self._parent.disconnect_button(self._clip_loop_start_inc_button, self._clip_loop_start_value)
	self._clip_loop_end_dec_button = self._parent.disconnect_button(self._clip_loop_end_dec_button, self._clip_loop_end_value)
	self._clip_loop_end_inc_button = self._parent.disconnect_button(self._clip_loop_end_inc_button, self._clip_loop_end_value)
	self._clip_loop_on_off_button = self._parent.disconnect_button(self._clip_loop_on_off_button, self._clip_loop_on_off_value)
	self._reinit_button = self._parent.disconnect_button(self._reinit_button, self._reinit_value)
	self._tempo_encoder = self._parent.disconnect_encoder(self._tempo_encoder, self._tempo_value)
	self.song().view.remove_detail_clip_listener(self.on_clip_changed)
	self.remove_clip_listeners()
	self._parent = None
	    
    
    def on_enabled_changed(self):
	self.update()
        

    def update(self):    
        self.on_clip_changed()

    
    def set_shift_button(self, button):
	self._shift_button = self._parent.set_button(button, self._shift_button, self._shift_value, False)
	
    
    def set_device_left_button(self, button):
	self._device_left_button = self._parent.set_button(button, self._device_left_button, self._device_nav_value, True)
    
	
    def set_device_right_button(self, button):
	self._device_right_button = self._parent.set_button(button, self._device_right_button, self._device_nav_value, True)
	
	
    def set_clip_loop_start_dec_button(self, button):
	self._clip_loop_start_dec_button = self._parent.set_button(button, self._clip_loop_start_dec_button, self._clip_loop_start_value, True)
	
    
    def set_clip_loop_start_inc_button(self, button):
	self._clip_loop_start_inc_button = self._parent.set_button(button, self._clip_loop_start_inc_button, self._clip_loop_start_value, True)
	
    
    def set_clip_loop_end_dec_button(self, button):
	self._clip_loop_end_dec_button = self._parent.set_button(button, self._clip_loop_end_dec_button, self._clip_loop_end_value, True)
	
	
    def set_clip_loop_end_inc_button(self, button):
	self._clip_loop_end_inc_button = self._parent.set_button(button, self._clip_loop_end_inc_button, self._clip_loop_end_value, True)
	
	
    def set_clip_loop_on_off_button(self, button):
	self._clip_loop_on_off_button = self._parent.set_button(button, self._clip_loop_on_off_button, self._clip_loop_on_off_value, False)
	
	
    def set_reinit_button(self, button):
	self._reinit_button = self._parent.set_button(button, self._reinit_button, self._reinit_value, False)
	
	
    def set_tempo_encoder(self, encoder):
	self._tempo_encoder = self._parent.set_encoder(encoder, self._tempo_encoder, self._tempo_value, False)

	
    def _shift_value(self, value):
	""" Set shift_pressed variable based on passed value. This modifies the resolution of the Clip Loop Start/End and Tempo functions. """
	assert (self._shift_button != None)
        assert (value in range(128))
	self._shift_pressed = value != 0
	    
	    
    def _device_nav_value(self, value, sender):
	""" Move left/right between devices on the selected track. """
	assert (self._device_left_button != None) and (self._device_right_button != None)
	assert isinstance(sender, ButtonElement)
        assert (value in range(128))
	if self.is_enabled() and value:
	    if not self.application().view.is_view_visible('Detail') or not self.application().view.is_view_visible('Detail/DeviceChain'):
		self.application().view.show_view('Detail')
		self.application().view.show_view('Detail/DeviceChain')
	    else:
		direction = Live.Application.Application.View.NavDirection.left
		if sender == self._device_right_button:
		    direction = Live.Application.Application.View.NavDirection.right
		self.application().view.scroll_view(direction, 'Detail/DeviceChain', False)
	    
	
    def _clip_loop_start_value(self, value, sender):
	""" Dec/inc clip loop start by 1 beat (1/16th of a beat when shift pressed). """
	assert (self._clip_loop_start_dec_button != None) and (self._clip_loop_start_inc_button != None)
	assert isinstance(sender, ButtonElement)
        assert (value in range(128))
	if self.is_enabled() and value and self._clip:
	    factor = -(self._parent._globals['CLIP_LOOP_RES'])
	    if self._shift_pressed:
		factor = -(self._parent._globals['CLIP_LOOP_SHIFT_RES'])
	    if sender == self._clip_loop_start_inc_button:
		factor = abs(factor)
	    self._clip.loop_start = max(0.0, min(self._clip.loop_end - factor, (self._clip.loop_start + factor)))
	    
	    
    def _clip_loop_end_value(self, value, sender):
	""" Dec/inc clip loop end by 1 beat (1/16th of a beat when shift pressed). """
	assert (self._clip_loop_end_dec_button != None) and (self._clip_loop_end_inc_button != None)
	assert isinstance(sender, ButtonElement)
        assert (value in range(128))
	if self.is_enabled() and value and self._clip:
	    factor = -(self._parent._globals['CLIP_LOOP_RES'])
	    if self._shift_pressed:
		factor = -(self._parent._globals['CLIP_LOOP_SHIFT_RES'])
	    if sender == self._clip_loop_end_inc_button:
		factor = abs(factor)
	    self._clip.loop_end = max((self._clip.loop_start - factor), (self._clip.loop_end + factor))
	    
	    
    def _clip_loop_on_off_value(self, value):
	""" Toggle clip loop on/off. """
	assert (self._clip_loop_on_off_button != None) 
        assert (value in range(128))
	if self.is_enabled() and value and self._clip:
	    self._clip.looping = not(self._clip.looping)
	    
	    
    def _reinit_value(self, value):
	""" Re-initialize entire script. """
	assert (self._reinit_button != None)
        assert (value in range(128))
	if self.is_enabled() and value:
	    self._parent.refresh_state()
	    self._parent.request_rebuild_midi_map()
	    
	    
    def _tempo_value(self, value):
	""" Dec/inc tempo in 1 BPM increments (or 0.1 BPM increments when shift pressed). """
	assert (self._tempo_encoder != None) 
        assert (value in range(128))
	if self.is_enabled():
	    factor = -(self._parent._globals['BPM_ENC_RES'])
	    if self._shift_pressed:
		factor = -(self._parent._globals['BPM_ENC_SHIFT_RES'])
	    if value < 64:
		factor = abs(factor)
	    self.song().tempo = max(20.0, min(self._parent._globals['BPM_MAX'], (self.song().tempo + float(factor))))
    
	    
    def on_looping_changed(self):
	if self.is_enabled() and self._clip and self._clip_loop_on_off_button:
	    if self._clip.looping:
		self._clip_loop_on_off_button.turn_on()
	    else:
		self._clip_loop_on_off_button.turn_off()
	    
	    
    def on_clip_changed(self):
	if self.is_enabled() and self._clip != self.song().view.detail_clip:
	    self.remove_clip_listeners()
	    self._clip = self.song().view.detail_clip
	    if self._clip:
		if not self._clip.looping_has_listener(self.on_looping_changed):
		    self._clip.add_looping_listener(self.on_looping_changed)
	    self.on_looping_changed()
		    
		    
    def remove_clip_listeners(self):
	if self._clip:
	    if self._clip.looping_has_listener(self.on_looping_changed):
		self._clip.remove_looping_listener(self.on_looping_changed)
	self._clip = None
	    
    
# local variables:
# tab-width: 4