"""
# Copyright (C) 2013-2016 Stray <stray411@hotmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For questions regarding this module contact
# Stray <stray411@hotmail.com>
"""

# emacs-mode: -*- python-*-
# -*- coding: utf-8 -*-

import Live 
from MixerComponent import MixerComponent 
from SMEQ import SMEQ

class SMMixer(MixerComponent):
    " Mixer component that includes Returns in its tracks_to_use, uses a modified TrackEQComponent and handles monitoring devices on the tracks it controls. "
    __module__ = __name__

    def __init__(self, parent, num_tracks):
	self._tracks_to_monitor = {}
	self._parent = parent
	MixerComponent.__init__(self, num_tracks)
        for index in range (num_tracks): 
	    self._track_eqs.append(SMEQ(parent))
	    self.register_components(self._track_eqs[index])
	self._reassign_tracks()
	
	
    def disconnect(self):
	MixerComponent.disconnect(self)
	self.remove_track_device_listeners()
	self._parent = parent = None
	
	
    def _reassign_tracks(self):
	""" Modified to remove/set up device listeners. """
	tracks = self.tracks_to_use()
        returns = self.song().return_tracks
	self.remove_track_device_listeners()
        for index in range(len(self._channel_strips)):
            track_index = (self._track_offset + index)
            track = None
            if (len(tracks) > track_index):
                track = tracks[track_index]
            self._channel_strips[index].set_track(track)
	    self.setup_track_device_listeners(track, index)
        for index in range(len(self._return_strips)):
            if (len(returns) > index):
                self._return_strips[index].set_track(returns[index])
            else:
                self._return_strips[index].set_track(None)
        if self._bank_down_button != None:
            if self._track_offset > 0:
                self._bank_down_button.turn_on()
            else:
                self._bank_down_button.turn_off()
        if self._bank_up_button != None:
            if len(tracks) > self._track_offset + len(self._channel_strips):
                self._bank_up_button.turn_on()
            else:
                self._bank_up_button.turn_off()
        return None
	    
    
    def setup_track_device_listeners(self, track, index):
	""" Setup device listeners for one of the channel strips being controlled. """
	assert track == None or isinstance(track, Live.Track.Track)
	assert isinstance(index, int)
	device_list = []
	if track:
	    self._tracks_to_monitor[index] = track
	    dev_listener = lambda index = index:self.on_track_devices_changed(index)
	    device_list = self.add_device_listeners(track, dev_listener)
	self.update_track_device_modules(device_list, index)
	
	
    def add_device_listeners(self, track, listener):
	""" Add device and chain listeners for the track. """
	assert track == None or isinstance(track, Live.Track.Track)
	assert (dir(listener).count('im_func') is 1) or (dir(listener).count('func_name') is 1)
	device_list = []
	if not track.devices_has_listener(listener):
	    track.add_devices_listener(listener)
	for device in track.devices:
	    device_list.append(device)
	    if device.can_have_chains:
		if not device.chains_has_listener(listener):
		    device.add_chains_listener(listener)
		for chain in device.chains:
		    if not chain.devices_has_listener(listener):
			chain.add_devices_listener(listener)
		    for device in chain.devices:
			device_list.append(device)
	return device_list
    
    
    def update_track_device_modules(self, device_list, index): 
	""" For now, this just passes new device list to SMEQs. """
	assert isinstance(device_list, list)
	assert isinstance(index, int)
	if (len(self._track_eqs) > index):
	    self._track_eqs[index].on_devices_changed(device_list)
	    
	    
    def on_track_devices_changed(self, index):
	""" Called when any devices change on one of the channel strips being controlled. """ 
	if self._tracks_to_monitor.has_key(index):
	    track = self._tracks_to_monitor[index]
	    if track:
		device_list = self.get_device_list(track)
		self.update_track_device_modules(device_list, index)
		
		
    def remove_track_device_listeners(self):
	for index in range(8):
	    if self._tracks_to_monitor.has_key(index):
		track = self._tracks_to_monitor[index]
		if track:
		    dev_listener = lambda index = index:self.on_track_devices_changed(index)
		    self.remove_device_listeners(track, dev_listener)
	self._tracks_to_monitor = {}
	
	
    def remove_device_listeners(self, track, listener):
	assert track == None or isinstance(track, Live.Track.Track)
	assert (dir(listener).count('im_func') is 1) or (dir(listener).count('func_name') is 1)
	if track:
	    if track.devices_has_listener(listener):
		track.remove_devices_listener(listener)
	    for device in track.devices:
		if device and device.can_have_chains:
		    if device.chains_has_listener(listener):
			device.remove_chains_listener(listener)
		    for chain in device.chains:
			if chain.devices_has_listener(listener):
			    chain.remove_devices_listener(listener)
	
	
    def get_device_list(self, track):
	""" Return list of devices on the track (one level of nesting). """
	assert track == None or isinstance(track, Live.Track.Track)
	device_list = []
	if track:
	    for device in track.devices:
		device_list.append(device)
		if device.can_have_chains:
		    for chain in device.chains:
			for device in chain.devices:
			    device_list.append(device)
	return device_list
	

    def tracks_to_use(self):
        """ Modified to include visible tracks and return tracks. """
        return tuple(self.song().visible_tracks) + tuple(self.song().return_tracks)
    
# local variables:
# tab-width: 4