"""
# Copyright (C) 2013-2016 Stray <stray411@hotmail.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# For questions regarding this module contact
# Stray <stray411@hotmail.com>
"""

# emacs-mode: -*- python-*-
# -*- coding: utf-8 -*-

import Live
from _Framework.SessionComponent import SessionComponent 
from _Framework.SceneComponent import SceneComponent
from _Framework.ClipSlotComponent import ClipSlotComponent
from _Framework.SubjectSlot import subject_slot
	
class SMSession(SessionComponent):
    __module__ = __name__
    __doc__ = " Modified SessionComponent for Live 9 that creates a modified SceneComponent. "

    def __init__(self, num_tracks, num_scenes):        
	SessionComponent.__init__(self, num_tracks, num_scenes)

	
    def _create_scene(self): 
        return SMScene(self._num_tracks, self.tracks_to_use)
    
    
class SMScene(SceneComponent):
    __module__ = __name__
    __doc__ = "  Modified SceneComponent that creates a modified ClipSlotComponent. "
    
    def __init__(self, num_slots, tracks_to_use_callback):
	SceneComponent.__init__(self, num_slots, tracks_to_use_callback)
	
	
    def _create_clip_slot(self):
        return SMClipSlot()
    
    
class SMClipSlot(ClipSlotComponent):
    __module__ = __name__
    __doc__ = " Modified ClipSlotComponent that includes a looping value in addition to the default clip-state values. "
    
    def __init__(self):
	ClipSlotComponent.__init__(self)
	self._looping_value = self._started_value
	
	
    def set_looping_value(self, value):
	""" Set the LED value to use for clips that are playing and looping. """
        assert isinstance(value, int)
        assert (value in range(128))
        self._looping_value = value
	
	
    def set_clip_slot(self, clip_slot): 
	""" Modified to add looping changed subject. """
	self._clip_slot = clip_slot
        self._update_clip_property_slots()
        self._on_slot_triggered_changed.subject = clip_slot
        self._on_slot_playing_state_changed.subject = clip_slot
        self._on_clip_state_changed.subject = clip_slot
        self._on_controls_other_clips_changed.subject = clip_slot
        self._on_has_stop_button_changed.subject = clip_slot
        self._on_clip_slot_color_changed.subject = clip_slot
        track = clip_slot.canonical_parent if clip_slot else None
        if track and track.can_be_armed:
            self._on_arm_value_changed.subject = track
            self._on_implicit_arm_value_changed.subject = track
        self.update()
	
	
    def _update_clip_property_slots(self):
	""" Modified to add looping changed subject. """
        clip = self._clip_slot.clip if self._clip_slot else None
        self._on_clip_playing_state_changed.subject = clip
        self._on_recording_state_changed.subject = clip
        self._on_clip_color_changed.subject = clip
	self._on_looping_changed.subject = clip
	
	
    def _feedback_value(self):
	""" Modified to send loop value if clip is playing and looping. """
        if self._clip_slot != None:
            track = self._clip_slot.canonical_parent
            slot_or_clip = self._clip_slot.clip if self.has_clip() else self._clip_slot
            if slot_or_clip.is_triggered:
                return self._triggered_to_record_value if slot_or_clip.will_record_on_start else self._triggered_to_play_value
            elif slot_or_clip.is_playing:
		if slot_or_clip.is_recording:
		    return self._recording_value
                return self._looping_value if slot_or_clip.looping else self._started_value
            elif slot_or_clip.color != None:
                return self._color_value(slot_or_clip.color)
            elif getattr(slot_or_clip, 'controls_other_clips', True) and self._stopped_value != None:
                return self._stopped_value
            elif self._track_is_armed(track) and self._clip_slot.has_stop_button:
                if self._record_button_value != None:
                    return self._record_button_value
		
    
    @subject_slot('looping')
    def _on_looping_changed(self):
        self.update()
	
	
# local variables:
# tab-width: 4
